from .crew_xUnit import xunit_generation, info_gatherer_crew
from .crew_gherkin import generate_gherkin
import argparse
import os
from dotenv import load_dotenv

def main() -> str:
    parser = argparse.ArgumentParser(description="Processa os parâmetros")
    parser.add_argument("generate", type=str, help="O que você deseja gerar (gherkin ou xunit)")
    parser.add_argument("--andes", type=str, help="Casos de usuário para gerar o Gherkin")
    parser.add_argument("--user_case", type=str, help="Casos de usuário para gerar o Gherkin")
    parser.add_argument("--feature", type=str, help="A feature para gerar o código xUnit")
    parser.add_argument("--dto_source", type=str, help="O caminho para os arquivos DTO")
    parser.add_argument("--swagger_path", type=str, help="O caminho para o documento Swagger")
    args = parser.parse_args()

    if args.generate == 'gherkin':
        if not args.user_case:
            parser.error("--user_case deve ser fornecido quando --generate é 'gherkin'")
        generate_gherkin(args.user_case)

    if args.generate == "xunit":
        if not args.feature:
            parser.error("--feature deve ser fornecida quando --generate é 'xunit'")

        # if not args.dto_source:
        #     parser.error("--dto_source deve ser fornecida quando --generate é 'xunit'")

        # if not args.swagger_path:
        #     parser.error("--swagger_path deve ser fornecida quando --generate é 'xunit'")

        swagger_path = os.getenv("SWAGGER_PATH")
        dto_source = os.getenv("DTO_SOURCE")
        xunit_generation(args.feature, swagger_path, dto_source)
    

if __name__ == "__main__":
    main()


from crewai import Agent, Task, Crew, Process, LLM
from dotenv import load_dotenv
from .utils import(
    init_task,
    init_agent, 
    init_llm,
    get_yaml_config
)
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path

def get_configs():
    config_path = Path(__file__).parent / 'config'
    yaml_config = get_yaml_config(base_path=config_path, files=["agents", "tasks", "output_examples"])
    agents_config = yaml_config["agents"]
    tasks_config = yaml_config["tasks"]
    outputs_examples = yaml_config["output_examples"]
    return agents_config, tasks_config, outputs_examples

llm_low_temp: LLM = init_llm()
llm_high_temp: LLM = init_llm(temp=0.6)

def crew_generation_gherkin(user_case: str) -> Crew:

    agents_config, tasks_config, outputs_examples = get_configs()

    crew_agents: dict[str, str] = agents_config["gherkin_crew"]
    crew_tasks: dict[str, str] = tasks_config["gherkin_tasks"]

    gherkin_writer: Agent = init_agent(crew_agents["gherkin_writer"], llm_high_temp)
    crew_tasks["gherkin_code"]["description"] = crew_tasks["gherkin_code"]["description"].format(user_case=user_case)
    gherkin_code: Task = init_task(
        crew_tasks["gherkin_code"],
        gherkin_writer,
        #output_file=f"etapas_geracao/rodada_{turn}.cs"
    )

    gherkin_reviewer: Agent = init_agent(crew_agents["gherkin_reviewer"], llm_low_temp)
    crew_tasks["gherkin_review"]["description"] = crew_tasks["gherkin_review"]["description"].format(user_case=user_case)
    gherkin_review: Task = init_task(
        crew_tasks["gherkin_review"],
        gherkin_reviewer,
        context=[gherkin_code],
        #output_file=f"etapas_geracao/revisao_{turn}.cs"
    )
    

    return Crew(
        agents=[gherkin_writer, gherkin_reviewer],
        tasks=[gherkin_code, gherkin_review],
        max_rpm=10,
        output_log_file="crew_log.txt",
        #manager_agent=manager_agent,
        process=Process.sequential,
        verbose=False
    )
    
def manager_crew(reviews: list[str]) -> Crew:
    agents_config, tasks_config, output_examples = get_configs()

    crew_agents: dict[str, str] = agents_config["gherkin_crew"]
    crew_tasks: dict[str, str] = tasks_config["gherkin_tasks"]

    crew_tasks["manager_gherkin_task"]["description"] = crew_tasks["manager_gherkin_task"]["description"] \
    .format(reviews[0], reviews[1], reviews[2])
    manager: Agent = init_agent(
        crew_agents["manager_gherkin"],
        llm=llm_low_temp,
    )
    final_task: Task = init_task(
        crew_tasks["manager_gherkin_task"],
        agent=manager,
        output_file="features/ListarBolsaFeature.feature"
    )

    return Crew(
        agents=[manager],
        tasks=[final_task],
        max_rpm=2,
        process=Process.sequential,
        verbose=False
    )

def generate_gherkin(user_case: str) -> None:
    crew_gherkin: Crew = crew_generation_gherkin(user_case)
    with ThreadPoolExecutor() as executor:
        runs = [executor.submit(crew_gherkin.kickoff) for _ in range(3)]
        results = [run.result() for run in runs]
    print(manager_crew(results).kickoff())


from crewai import Agent, Task, Crew, Process, LLM
from crewai_tools import FileReadTool, DirectoryReadTool
from typing import Dict, List
from .utils import (
    init_task,
    init_agent,
    init_llm, 
    get_yaml_config,
)
from concurrent.futures import ThreadPoolExecutor
from .extract_endpoints import extract_endpoints
import os
from pathlib import Path
from dotenv import load_dotenv


llm_low_temp: LLM = init_llm()

def get_configs():
    config_path = Path(__file__).parent / 'config'
    yaml_config = get_yaml_config(base_path=config_path, files=["agents", "tasks", "output_examples"])
    agents_config = yaml_config["agents"]
    tasks_config = yaml_config["tasks"]
    outputs_examples = yaml_config["output_examples"]
    return agents_config, tasks_config, outputs_examples

def crew_xunit_debate(feature: str, strings: Dict[str, str]) -> str:
    
    agents_dict: Dict[str, str] = strings["agents"]
    tasks_dict: Dict[str, str] = strings["tasks"]
    
    tasks: List[Task] = []
    agents: List[Agent] = []

    gemini_llm: LLM = init_llm(temp=0.2)

    csharp_xunit_writer_agent: Agent = init_agent(
        agents_dict["csharp_xunit_writer"],
        gemini_llm
    )

    tasks_dict["xunit_code_proposal"]["description"] = tasks_dict["xunit_code_proposal"]["description"].format(feature=feature)
    xunit_code_proposal: Task = init_task(tasks_dict["xunit_code_proposal"], agent=csharp_xunit_writer_agent)
    
    agents.append(csharp_xunit_writer_agent)
    tasks.append(xunit_code_proposal)

    for i in range(1,4):
        xunit_solution_discussion_agent: Agent = init_agent(agents_dict["xunit_solution_discussion"], gemini_llm)

        tasks_dict["debate"]["description"] = tasks_dict["debate"]["description"].format(feature=feature)
        debate: Task = init_task(tasks_dict["debate"], agent=xunit_solution_discussion_agent)

        agents.append(xunit_solution_discussion_agent)
        tasks.append(debate)
    
    result_analysis_manager_agent: Agent = init_agent(agents_dict["result_analysis_manager"], llm=gemini_llm)

    tasks_dict["manager_xunit_task"]["description"] = tasks_dict["manager_xunit_task"]["description"].format(feature=feature)
    manager_xunit_task: Task = init_task(
        tasks_dict["manager_xunit_task"],
        output_file=f"modalidade_bolsa_crew.cs",
        agent=result_analysis_manager_agent,
        context=[tasks[-1]],
    )

    crew: Crew = Crew(
        agents=agents + [result_analysis_manager_agent],
        tasks=tasks + [manager_xunit_task],
        process=Process.sequential,
        verbose=True
    )

    return crew.kickoff().raw

def info_gatherer_crew(feature: str, swagger_path: str, dto_source: str) -> tuple[str, str]:
    agents_config, tasks_config, output_examples = get_configs()

    endpoints = extract_endpoints(swagger_path)

    crew_agents: dict[str, str] = agents_config["info_gatherer_crew"]
    crew_tasks: dict[str, str] = tasks_config["info_gatherer_tasks"]

    crew_tasks["api_url_find"]["description"] = crew_tasks["api_url_find"]["description"].format(feature=feature, api_endpoints=endpoints)
    crew_tasks["dto_file_find"]["description"] = crew_tasks["dto_file_find"]["description"].format(feature=feature, dto_source=dto_source)

    #api find
    agent_api_finder = init_agent(
        config=crew_agents["agent_api_finder"],
        llm=llm_low_temp,
        tools=[FileReadTool()],
    )
    api_url_find = init_task(
        config=crew_tasks["api_url_find"],
        agent=agent_api_finder,
    )
    #

    #dto find
    agent_file_searcher = init_agent(
        config=crew_agents["agent_file_searcher"],
        llm=llm_low_temp,
        tools=[DirectoryReadTool(), FileReadTool()]
    )
    dto_file_find = init_task(
        config=crew_tasks["dto_file_find"],
        agent=agent_file_searcher,
    )
    #

    crew = Crew(
        agents=[agent_api_finder, agent_file_searcher],
        tasks=[dto_file_find, api_url_find],
        verbose=False,
        process=Process.sequential
    )

    crew.kickoff()

    return dto_file_find.output.raw, api_url_find.output.raw

def crew_xunit_generation(feature: str, api_url: str, dto_code: str) -> Crew:
    gemini_llm: LLM = init_llm(temp=0.2)
    agents_config, tasks_config, outputs_examples = get_configs()
    
    crew_agents: dict[str, str] = agents_config["xunit_crew"]
    crew_tasks: dict[str, str] = tasks_config["xunit_tasks"]

    ###
    #bind das features e concantenacao com o output de exemplo
    crew_tasks["xunit_write"]["description"] = \
        crew_tasks["xunit_write"]["description"] \
        .format(feature, dto_code, api_url) + \
        outputs_examples[crew_tasks["xunit_write"]["output_example"]]

    crew_tasks["xunit_review"]["description"] = \
        crew_tasks["xunit_review"]["description"] \
        .format(feature) + \
        outputs_examples[crew_tasks["xunit_review"]["output_example"]]
    ###
    
    #write xunit
    xunit_writer: Agent = init_agent(
        config=crew_agents["xunit_writer"],
        llm=gemini_llm
        )
    xunit_write: Task = init_task(
        config=crew_tasks["xunit_write"],
        agent=xunit_writer
        )

    #xunit review
    xunit_reviewer: Agent = init_agent(
        config=crew_agents["xunit_reviewer"],
        llm=gemini_llm
    )
    xunit_review: Task = init_task(
        config=crew_tasks["xunit_review"],
        agent=xunit_reviewer,
        context=[xunit_write],
        )
    

    return Crew(
        agents=[xunit_writer, xunit_reviewer],
        tasks=[xunit_write, xunit_review],
        output_log_file="crew_log.txt",
        process=Process.sequential,
        verbose=False
        )

def manager_crew(reviews: tuple[str]) -> Crew:
    agents_config, tasks_config, outputs_examples = get_configs()

    crew_agents: dict[str, str] = agents_config["xunit_crew"]
    crew_tasks : dict[str, str] = tasks_config["xunit_tasks"]

    #bind da feature e output example
    crew_tasks["manager_xunit_task"]["description"] = \
        crew_tasks["manager_xunit_task"]["description"].format(reviews[0], reviews[1], reviews[2]) + \
        outputs_examples[crew_tasks["manager_xunit_task"]["output_example"]]
    #

    #manager
    manager: Agent = init_agent(
        config=crew_agents["result_analysis_manager"],
        llm=llm_low_temp
    )
    manager_task: Task = init_task(
        config=crew_tasks["manager_xunit_task"],
        agent=manager,
        output_file="xUnit/codigo.cs"
    )
    #

    return Crew(
        agents=[manager],
        tasks=[manager_task],
        process=Process.sequential,
        verbose=False
    )

def xunit_generation(feature: str, swagger_path: str, dto_source: str) -> str:
    dto_code, api_url = info_gatherer_crew(feature, swagger_path, dto_source)
    crew_xunit: Crew = crew_xunit_generation(feature, api_url, dto_code)
    with ThreadPoolExecutor() as executor:
        runs = [executor.submit(crew_xunit.kickoff) for _ in range(3)]
        results = [run.result() for run in runs]
    print(manager_crew(results).kickoff().raw)



from crewai import LLM, Agent, Task
import os
from typing import Dict, Union, TypedDict, Callable, Any
from dotenv import load_dotenv
from yaml import safe_load
import asyncio

class UserCaseDict(TypedDict):
    user_case: str

class FeatureDict(TypedDict):
    feature: str

def run_crew_async(func: Callable, **params: Any) -> str:
    return asyncio.run(func(params))

def init_llm(config = {}, temp = 0.0, api_key = os.getenv("API_KEY"), model = os.getenv("LLM_MODEL")) -> LLM:
    return LLM(
        model=config.get("model", model),
        temperature=config.get("temperature", temp),
        api_key=config.get("api_key", api_key),
        timeout=config.get("timeout", None),
        max_tokens=config.get("max_tokens", None),
        top_p=config.get("top_p", None),
        frequency_penalty=config.get("frequency_penalty", None),
        presence_penalty=config.get("presence_penalty", None),
        response_format=config.get("response_format", None),
        seed=config.get("seed", None)
    )

def init_agent(config: dict[str, str], llm , tools=[]) -> Union[Agent, None]:
    if "role" not in config: 
        print("Agent role must be defined")
        return
    if "goal" not in config:
        print("Agent goal must be defined")
        return
    if "backstory" not in config: 
        print("Agent backstory must be defined")
        return 
    return Agent(
        role = config.get("role"),
        goal = config.get("goal"),
        backstory= config.get("backstory"),
        llm = llm,
        config = None,
        cache = config.get("cache", True),
        verbose = config.get("verbose", False),
        max_rpm = config.get("max_rpm", None),
        allow_delegation = config.get("allow_delegation", False),
        tools = tools,
        max_iter = config.get("max_iter", 25),
        function_calling_llm = config.get("function_calling_llm", None),
        max_execution_time = config.get("max_execution_time", None),
        step_callback = config.get("step_callback", None),
        system_template = config.get("system_template", None),
        prompt_template = config.get("prompt_template", None),
        response_template = config.get("response_template", None),
        allow_code_execution = config.get("allow_code_execution", False),
        max_retry_limit = config.get("max_retry_limit", 2),
        use_system_prompt = config.get("use_system_prompt", True),
        respect_context_window = config.get("respect_context_window", True),
        code_execution_mode = config.get("code_execution_mode", 'safe'),
    )

def init_task(config: Dict[str, str], agent: Agent = None, context=None, tools=[], output_file="") -> Union[Task, None]:
    if "description" not in config:
        print("Task description must be defined")
        return
    if "expected_output" not in config:
        print("Task expected_output must be defined")
        return

    return Task(
        description=config.get("description"),
        expected_output=config.get("expected_output"),
        agent=agent,
        tools=tools,
        async_execution = config.get("async_execution", False),
        context=context,
        config = None,
        output_json = config.get("output_json", None),
        output_pydantic = config.get("output_pydantic", None),
        output_file = output_file,
        human_input = config.get("human_input", False),
        converter_cls = config.get("converter_cls", None),
        callback = config.get("callback", None)
    )

def read_yaml(file_path: str) -> Dict[str, str]:
    with open(file_path) as file:
        return safe_load(file)

def get_yaml_config(base_path: str, files=["agents", "tasks"]) -> Dict[str, Dict[str, str]]:
    dict_return: Dict[str, Dict] = {}
    for file in files:
        dict_return[file] = read_yaml(f"{base_path}/{file}.yaml")
    return dict_return

# def init_agent(
#     agent_profile: AgentProfile,
#     llm: LLM,
#     config = None,
#     cache = True,
#     verbose = False,
#     max_rpm = None,
#     allow_delegation = False,
#     tools = [],
#     max_iter = 25,
#     function_calling_llm = None,
#     max_execution_time = None,
#     step_callback = None,
#     system_template = None,
#     prompt_template = None,
#     response_template = None,
#     allow_code_execution = False,
#     max_retry_limit = 2,
#     use_system_prompt = True,
#     respect_context_window = True,
#     code_execution_mode = 'safe'
# ) -> Agent:
#     return Agent(
#         role = agent_profile["role"],
#         goal = agent_profile["goal"],
#         backstory = agent_profile["backstory"],
#         llm = llm,
#         config = config,
#         cache = cache,
#         verbose = verbose,
#         max_rpm = max_rpm,
#         allow_delegation = allow_delegation,
#         tools = tools,
#         max_iter = max_iter,
#         function_calling_llm = function_calling_llm,
#         max_execution_time = max_execution_time,
#         step_callback = step_callback,
#         system_template = system_template,
#         prompt_template = prompt_template,
#         response_template = response_template,
#         allow_code_execution = allow_code_execution,
#         max_retry_limit = max_retry_limit,
#         use_system_prompt = use_system_prompt,
#         respect_context_window = respect_context_window,
#         code_execution_mode = code_execution_mode
#     )

# def init_llm(model: str = os.getenv("DEFAULT_LLM_MODEL"), temp: float = 0.0, key: str = os.getenv("GOOGLE_API_KEY")) -> LLM:    
    
#     return LLM(
#         model=model,
#         temperature=temp,
#         api_key=key,
#     )

# def init_task(
#     task_profile,
#     agent,
#     tools = [],
#     async_execution = False,
#     context = None,
#     config = None,
#     output_json = None,
#     output_pydantic = None,
#     output_file = "",
#     human_input = False,
#     converter_cls = None,
#     callback = None
# ) -> Task:
#     bind_output_example(task_profile["description"], task_profile["output_example"])
#     return Task(
#         description = task_profile["description"],
#         expected_output = task_profile["expected_output"],
#         agent = agent,
#         tools = tools,
#         async_execution = async_execution,
#         context = context,
#         config = config,
#         output_json = output_json,
#         output_pydantic = output_pydantic,
#         output_file = output_file,
#         human_input = human_input,
#         converter_cls = converter_cls,
#         callback = callback
#     )

# def bind_output_example(*tasks) -> None:
#     print(tasks)
    #task["description"] = task["description"] + task["output_example"]

# def init_crew(agents, tasks):
#     print(agents, tasks)

#print(os.getcwd())


import json

def extract_endpoints(swagger_path: str) -> list[str]:
    with open(swagger_path) as f:
        swagger_data = json.load(f)

    # Obtenha os endpoints
    endpoints = ""
    for path, methods in swagger_data.get('paths', {}).items():
        for method in methods.keys():
            endpoints += f"{method.upper()} {path}\n"

    return endpoints


import re
import json

def load_input_from_file(file_path):
    """Função para carregar o conteúdo do arquivo."""
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            return file.read()
    except Exception as error:
        raise RuntimeError(f"Erro ao ler o arquivo: {error}")

def extract_events(input_text):
    """Função para extrair todos os eventos."""
    usecase_pattern = re.compile(
        r"usecase\s+(\w+)\s*{\s*name:\s*\"([^\"]+)\"\s*.*?description:\s*\"([^\"]+)\"\s*.*?performer:\s*(\w+)\s*.*?(event.*?})\s*}",
        re.DOTALL
    )
    event_pattern = re.compile(
        r"event\s+(\w+)\s*{\s*name:\s*\"([^\"]+)\"\s*.*?description:\s*\"([^\"]+)\"\s*.*?action:\s*\"([^\"]+)\"\s*}",
        re.DOTALL
    )

    events = []

    for usecase_match in usecase_pattern.finditer(input_text):
        events_block = usecase_match.group(5)

        for event_match in event_pattern.finditer(events_block):
            event_id = event_match.group(1)
            event_name = event_match.group(2)
            event_description = event_match.group(3)
            event_action = event_match.group(4).strip()

            events.append({
                "event_id": event_id,
                "name": event_name,
                "description": event_description,
                "action": event_action
            })

    return events

# Caminho para o arquivo de entrada

def get_events(file_path: str):
    input_text = load_input_from_file(file_path)
    events = extract_events(input_text)
    return events


from setuptools import setup, find_packages

setup(
    name="test_ai_leds",
    version="0.1.1",
    author="LEDS",
    author_email="gabrieldpbrunetti@gmail.com",
    description="AI automated test generation" ,
    packages=find_packages(include=['testailib']),
    include_package_data=True,
    install_requires=[
        "crewai",
        "crewai_tools",
        "google_generativeai",
        "pyaml"
    ],
    entry_points={
        "console_scripts": [
            "testai=testailib.main:main",  # Executável que chama a função main
        ]
    }
)